function f = KF_Reduced_Form_MA_loglik(paramKF,y)
%% Estimate a reduced-form MA(q) model using Kalman Filtering 
% Model: MA(q), r = 1+q
% y_t (1x1)   =  C (1xr) x_t (rx1)
% x_t (rx1)   =  A (rxr) x_{t-1} (rx1)  +   B (rx1) u_t (1x1)
% x_t (rx1)   =  [eps_t, eps_{t-1}, ..., eps_{t-q}]
% A (rxr)     =  [zeros(1,r);[eye(r-1),zeros(r-1,1)]];
% B (rx1)     =  [s2;zeros(r-1,1)];
% C (rx1)     =  [1, theta'];
% u_t (1x1)   =  Normal(0, 1);
% Parameters: s2, theta

n     = length(y);
s2    = paramKF(1);
theta = paramKF(2:end);
q     = length(theta);
r     = q+1;


if (s2>0)
    A     = [zeros(1,r);[eye(r-1),zeros(r-1,1)]];
    B     = [sqrt(s2);zeros(r-1,1)];
    C     = [1, theta'];

    xhat  = zeros(r,n);
    Pmat  = zeros(r,r,n);
    nu    = zeros(1,n);
    Fmat  = zeros(1,n);
    Kmat  = zeros(r,n);

    xhat(:,1)    =  [y(1);zeros(r-1,1)];
    Pmat(:,:,1)  =  s2*eye(r); 
    nu(1,1)      =  y(1)-C*xhat(:,1);
    Fmat(1,1)=C*Pmat(:,:,1)*C';
    Kmat(:,1)=A*Pmat(:,:,1)*C'/Fmat(1,1);

    for t=2:(n)
        xhat(:,t)  = A*xhat(:,t-1)+Kmat(:,t-1)*nu(:,t-1);

        Pmat(:,:,t)= (A*Pmat(:,:,t-1)*A')-(Kmat(:,t-1)*Kmat(:,t-1)')*Fmat(1,t-1)+B*B';
   
        nu(1,t)    = y(t)-C*xhat(:,t);
        Fmat(1,t)  = C*Pmat(:,:,t)*C';       
        Kmat(:,t)  = A*Pmat(:,:,t)*C'/Fmat(1,t);
        
    end;


    if  sum(Fmat<0)==0
    
        temp  = - n*log(2*pi)/2 - 1/2*sum(log(Fmat)) -sum(nu.^2./(Fmat))/2; 
    
        f = - temp/n; % don't forget the minus to get a max!
    else
        
        f = 10^7;
    
    end;
    
else
    f = 10^7; 
end 
return;
