%% This Version: May 18, 2021. @copyright Rui Da and Dacheng Xiu
% If you use these codes, please cite our paper 
% "When Moving-Average Models Meet High-Frequency Data: Uniform Inference on Volatility." (2021)

function results = QMLE_MA(y,T,qtrue,quse)

%% Estimate Parameters using KF_Reduced_Form_MA_loglik
%  y: raw returns
%  T: time window (annualized)
%  qtrue: alpha
%  quse: a particular likelihood to compare, e.g., quse = 0 is likelihood based on i.i.d. noise.
%  output corresponding to [AIC, AIC*, i.i.d. noise]

 
    n = length(y);

    qlist   = [0,1,2,3,4,5,6,7,8,9,10,11,12];

    tempQML_q    = zeros(1,length(qlist));
    tempQML_var  = zeros(1,length(qlist));
    tempQML_iota = zeros(1,length(qlist));
    tempQML_flag = zeros(1,length(qlist));
    tempBIC      = zeros(1,length(qlist));
    tempAIC      = zeros(1,length(qlist));
    tempQML_fval = zeros(1,length(qlist));
    tempQML_param= cell(1,length(qlist));
    tempQML_gamma= cell(1,length(qlist));
%% Estimation

    paramRMstart_pre = [0.09];
    for ql = 1:length(qlist)
       
        if (qlist(ql)>max(qtrue,2) & tempBIC(1,qlist(ql)-1)<= tempBIC(1,qlist(ql)-2) & ~isnan(tempBIC(1,qlist(ql)))) |  qlist(ql)<=max(qtrue,2)
       
            % Find Initial Values for Reduced-Form Model       
            
            paramRMstart = find_initial_val(qlist(ql)+1,y); % raw return is an MA(q+1)
            
            % Select initial values for optimization 
            
            [paramRM,fval,exitflag,output] = fminsearch('KF_Reduced_Form_MA_loglik_C', paramRMstart, optimset('Display','off','TolX',1e-12,'TolFun',1e-12),y);
        
            [paramRM2,fval2, exitflag2,output2] = fminsearch('KF_Reduced_Form_MA_loglik_C', [paramRMstart_pre;0], optimset('Display','off','TolX',1e-12,'TolFun',1e-12),y);
            
            if fval>fval2
               
                paramRM=paramRM2;fval=fval2;exitflag=exitflag2;output=output2;
                
            end
            % Recover the Structural Parameters (Multiplicity or No Solution)
        
            [paramGamma,paramSM]   =   R2S(paramRM,T,n);  % paramRM is (q+2) vector
            
            tempQML_gamma{1,ql} = paramGamma;
            tempQML_param{1,ql} = paramSM(3:end);
            tempQML_var(1,ql)   = paramSM(1);
            tempQML_iota(1,ql)  = paramSM(2);
            tempQML_flag(1,ql)  = exitflag;
            tempQML_fval(1,ql)  = fval;
            tempQML_q(1,ql)     = qlist(ql); 
            
        
            tempBIC(1,ql) = 2*(fval)+(qlist(ql))/n;  % reserve this for AIC* in the paper. If BIC is needed, use instead 2*(fval)+(qlist(ql))*log(n)/n;  % BIC 

            tempAIC(1,ql) = 2*(fval)+(qlist(ql))/n;  % AIC
     
        
        else
            
            tempQML_gamma{1,ql} = nan(1);
            tempQML_param{1,ql} = nan(1);
            tempQML_var(1,ql)   = nan(1);
            tempQML_iota(1,ql)  = nan(1);
            tempQML_flag(1,ql)  = nan(1);
            tempQML_fval(1,ql)  = nan(1);
            tempBIC(1,ql)       = nan(1); 
            tempAIC(1,ql)       = nan(1); 
            tempQML_q(1,ql)     = nan(1); 
            
        end;
       paramRMstart_pre = paramRMstart;
    
    end;

%%

    results.BIC       =    min(tempBIC);
    results.AIC       =    min(tempAIC);
    ka                =    find(tempAIC == results.AIC,1);
    kb                =    max(find(tempBIC == results.BIC,1),find(qlist == qtrue));%This is AIC*. If BIC is needed, use instead: find(tempBIC == results.BIC,1);%
    ktrue            =     find(qlist == quse);
    results.iv        =    [tempQML_var(1,ka),tempQML_var(1,kb),tempQML_var(1,ktrue)];
    results.iota      =    [tempQML_iota(1,ka),tempQML_iota(1,kb),tempQML_iota(1,ktrue)];
    results.fval      =    [tempQML_fval(1,ka),tempQML_fval(1,kb),tempQML_fval(1,ktrue)];
    results.BIC_FULL  =    tempBIC;
    results.AIC_FULL  =    tempAIC;
    qa                =    tempQML_q(1,ka);
    qb                =    tempQML_q(1,kb);
    results.q         =    [qa,qb,quse];
    results.gamma     =    zeros(qlist(end)+1,3);
    results.ma        =    nan(qlist(end),3);
    results.rho       =    zeros(qlist(end),3);
    
    results.ma(1:qa,1)   =    tempQML_param{1,ka}(1:qa);
    results.ma(1:qb,2)   =    tempQML_param{1,kb}(1:qb);
    results.ma(1:quse,3)=    tempQML_param{1,ktrue}(1:quse);
    
    results.gamma(1:qa+1,1) = tempQML_gamma{1,ka}(1:qa+1);
    results.gamma(1:qb+1,2) = tempQML_gamma{1,kb}(1:qb+1);
    results.gamma(1:quse+1,3) = tempQML_gamma{1,ktrue}(1:quse+1);
    
    results.rho(1:qa,1)     = results.gamma(2:qa+1,1)./results.gamma(1,1);
    results.rho(1:qb,2)     = results.gamma(2:qb+1,2)./results.gamma(1,2);
    results.rho(1:quse,3)  = results.gamma(2:quse+1,3)./results.gamma(1,3);
    
    results.n               = n;
         
    
    % AIC/BIC robust stds
    [C4,D4,B1,B2,B3]  =   quarticity_func_general(cumsum(y),T,0.8);

    results.quarticity =  [C4,D4,B1,B2,B3]; 
    
    B_a                =   min(abs(1./(results.iv(1))./(results.gamma(1,1) - results.gamma(2,1))*(B1 + B2)+1./(results.gamma(1,1) - results.gamma(2,1))^2*B3),log(n));
    ra_a               =   n.^(-0.5)*(5.*T^(-1).*(C4+D4).*(results.iv(1)).^(-0.5)+T^(-1).*(results.iv(1)).^(1.5).*B_a)...
                             .*(T*abs(sum(2*results.gamma(2:qa+1,1))+results.gamma(1,1))).^(0.5);
                    
                        
    B_b                =   min(abs(1./(results.iv(2))./(results.gamma(1,2) - results.gamma(2,2))*(B1 + B2)+1./(results.gamma(1,2) - results.gamma(2,2))^2*B3),log(n));
    ra_b               =   n.^(-0.5)*(5.*T^(-1).*(C4+D4).*(results.iv(2)).^(-0.5)+T^(-1).*(results.iv(2)).^(1.5).*B_b)...
                             .*(T*abs(sum(2*results.gamma(2:qb+1,2))+results.gamma(1,2))).^(0.5);
                           
    %    
    B_c                =    min(abs(1./(results.iv(3))./(results.gamma(1,3) - results.gamma(2,3))*(B1 + B2)+1./(results.gamma(1,3) - results.gamma(2,3))^2*B3),log(n));
    ra_c               =   n.^(-0.5)*(5.*T^(-1).*(C4+D4).*(results.iv(3)).^(-0.5)+T^(-1).*(results.iv(3)).^(1.5).*B_c)...
                           .*(T*abs(sum(2*results.gamma(2:qtrue+1,3))+results.gamma(1,3))).^(0.5); 
                      
    results.std        =   [sqrt(ra_a ), sqrt(ra_b),sqrt(ra_c)]; 
        
    rs_a               =    (T/n*(C4+D4)/T*(6+4*qa));
    rs_b               =    (T/n*(C4+D4)/T*(6+4*qb));
    rs_c               =    (T/n*(C4+D4)/T*(6+4*qtrue));
    results.sstd       =    [sqrt(rs_a), sqrt(rs_b), sqrt(rs_c)];  
    

    
    results.ustd        = sqrt(results.std.^2 + results.sstd.^2);
    results.B           = [B_a,B_b,B_c];
    
  
