%% This code produces Table 1 and Figure 1 in the paper.
% We run Simulation_Final(1..100) on a cluster. 
% The outputs from the server are collected in three separate subfolders: 
% Results_100, Results_500, and Results_2500,
% each with 100 files. 

clear;
close all;
KK = 2;
figure;
tblfinal = [];
%% Small noise results

pname =  'Results_100';
files = dir(['./',pname,'/*.mat']);

QML_var = zeros(1000,3);
QML_qhat= zeros(1000,3);

QML_std = zeros(1000,3);
QML_sstd = zeros(1000,3);
QML_ustd = zeros(1000,3);

RV_true = zeros(1000,1);
RV_hat = zeros(1000,1);
RVsub_hat = zeros(1000,1);
qhats = zeros(1000,3);
PVG_var = zeros(1000,27);
RK_var = zeros(1000,27);

for j=1:100

    y = load(['./',pname,'/',files(j).name]);
    
    QML_var  = QML_var + y.QML_var;
    QML_qhat = QML_qhat + y.QML_qhat;   
    QML_std = QML_std + y.QML_std;   
    QML_sstd = QML_sstd + y.QML_sstd; 
    QML_ustd = QML_ustd + y.QML_ustd; 
   
    RV_true = RV_true + y.RV_true;
    RVsub_hat = RVsub_hat + y.RVsub_hat;
    RV_hat = RV_hat + y.RV_hat;
    qhats = qhats+y.QML_qhat;    
    PVG_var = PVG_var+ y.PVG_var;
    RK_var = RK_var + y.RK_var;
end

T = y.T;
n=y.n;
M = y.M;


 subplot(3,3,1)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts   =   hist((QML_var(:,KK)-RV_true/T)./(QML_std(:,KK)),binCtrs);    
 nn      =   M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('small noise, using (i)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 
 subplot(3,3,7)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts=hist((QML_var(:,KK)-RV_true/T)./(QML_ustd(:,KK)),binCtrs);
 nn=M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('small noise, using (iii)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 
 subplot(3,3,4)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts=hist((QML_var(:,KK)-RV_true/T)./(QML_sstd(:,KK)),binCtrs); 
 nn=M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('small noise, using (ii)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 
 knl = y.knl;
 hnl= y.hnl;
 knpl = y.knpl;
 rkn = y.rkn;
 jkn = y.jkn;
 hkn = y.hkn;

 tbl = [mean(QML_var(:,1)-RV_true/T),...
    sqrt(var(QML_var(:,1)-RV_true/T)+(mean(QML_var(:,1)-RV_true/T))^2),...
    mean(QML_var(:,2)-RV_true/T),...
    sqrt(var(QML_var(:,2)-RV_true/T)+(mean(QML_var(:,2)-RV_true/T))^2),...
    mean(RVsub_hat(:,1)/T-RV_true/T),...
    sqrt(var(RVsub_hat(:,1)/T-RV_true/T)+(mean(RVsub_hat(:,1)/T-RV_true/T))^2),...
    mean(QML_var(:,3)-RV_true/T),...
    sqrt(var(QML_var(:,3)-RV_true/T)+(mean(QML_var(:,3)-RV_true/T))^2),...
    mean(RV_hat(:,1)/T-RV_true/T),...
    sqrt(var(RV_hat(:,1)/T-RV_true/T)+(mean(RV_hat(:,1)/T-RV_true/T))^2),... 
 ];
 tblrk = tbl;

 for k=1:27
    tbl = [tbl,...
        mean(PVG_var(:,k)-RV_true/T),...
        sqrt(var(PVG_var(:,k)-RV_true/T)+(mean(PVG_var(:,k)-RV_true/T))^2) ];
 end


 for k=1:27
    tblrk = [tblrk,...
        mean(RK_var(:,k)-RV_true/T),...
        sqrt(var(RK_var(:,k)-RV_true/T)+(mean(RK_var(:,k)-RV_true/T))^2) ];
 end

 tblfnl = [tbl(:,[1:2:end]);tbl(:,[2:2:end])]; 
 tblfnlrk = [tblrk(:,[1:2:end]);tblrk(:,[2:2:end])]; 
 idx = find(tblfnl(2,6:end) == min(tblfnl(2,6:end)));
 idxrk = find(tblfnlrk(2,6:end) == min(tblfnlrk(2,6:end)));
 tblout = [tblfnl(:,[1,2,idx+5]),tblfnlrk(:,idxrk+5),tblfnl(:,[3,4,5])];
 tblout=tblout'*100;
 tblfinal = [tblfinal, tblout];
 
 disp('PVG:')
 disp(['kn: ',num2str(knl(idx))]);
 disp(['hn: ',num2str(hnl(idx))]);
 disp(['knp: ',num2str(knpl(idx))]);

 disp('RK:')
 disp(['rkn: ',num2str(rkn(idxrk))]);
 disp(['jkn: ',num2str(jkn(idxrk))]);
 disp(['hkn: ',num2str(hkn(idxrk))]);


 %% median noise results
 
pname =  'Results_500';
files = dir(['./',pname,'/*.mat']);

QML_var = zeros(1000,3);
QML_qhat= zeros(1000,3);
QML_std = zeros(1000,3);
QML_sstd = zeros(1000,3);
QML_ustd = zeros(1000,3);
QML_std_true = zeros(1000,3);
QML_sstd_true = zeros(1000,3);
QML_ustd_true = zeros(1000,3);
RV_true = zeros(1000,1);
RV_hat = zeros(1000,1);
RVsub_hat = zeros(1000,1);
PVG_var = zeros(1000,27);
RK_var = zeros(1000,27);

for j=1:100

    
    y = load(['./',pname,'/',files(j).name]);
    
    QML_var  = QML_var + y.QML_var;
    QML_qhat = QML_qhat + y.QML_qhat;    
    QML_std = QML_std + y.QML_std;   
    QML_sstd = QML_sstd + y.QML_sstd; 
    QML_ustd = QML_ustd + y.QML_ustd; 

    RV_true = RV_true + y.RV_true;
    RV_hat = RV_hat + y.RV_hat;
    RVsub_hat = RVsub_hat + y.RVsub_hat;
    PVG_var = PVG_var+ y.PVG_var;
    RK_var = RK_var + y.RK_var;

end

T = y.T;
n=y.n;
M = y.M;





 subplot(3,3,2)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts   =   hist((QML_var(:,KK)-RV_true/T)./(QML_std(:,KK)),binCtrs);    
 nn      =   M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('median noise, using (i)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 
 subplot(3,3,8)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts=hist((QML_var(:,KK)-RV_true/T)./(QML_ustd(:,KK)),binCtrs);
 nn=M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('median noise, using (iii)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 
 subplot(3,3,5)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts=hist((QML_var(:,KK)-RV_true/T)./(QML_sstd(:,KK)),binCtrs); 
 nn=M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('median noise, using (ii)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 knl = y.knl;
 hnl= y.hnl;
 knpl = y.knpl;
 rkn = y.rkn;
 jkn = y.jkn;
 hkn = y.hkn;

tbl = [mean(QML_var(:,1)-RV_true/T),...
    sqrt(var(QML_var(:,1)-RV_true/T)+(mean(QML_var(:,1)-RV_true/T))^2),...
    mean(QML_var(:,2)-RV_true/T),...
    sqrt(var(QML_var(:,2)-RV_true/T)+(mean(QML_var(:,2)-RV_true/T))^2),...
    mean(RVsub_hat(:,1)/T-RV_true/T),...
    sqrt(var(RVsub_hat(:,1)/T-RV_true/T)+(mean(RVsub_hat(:,1)/T-RV_true/T))^2),...
    mean(QML_var(:,3)-RV_true/T),...
    sqrt(var(QML_var(:,3)-RV_true/T)+(mean(QML_var(:,3)-RV_true/T))^2),...
    mean(RV_hat(:,1)/T-RV_true/T),...
    sqrt(var(RV_hat(:,1)/T-RV_true/T)+(mean(RV_hat(:,1)/T-RV_true/T))^2),... 
];
tblrk = tbl;

for k=1:27
 tbl = [tbl,...
        mean(PVG_var(:,k)-RV_true/T),...
        sqrt(var(PVG_var(:,k)-RV_true/T)+(mean(PVG_var(:,k)-RV_true/T))^2) ];
end


for k=1:27
 tblrk = [tblrk,...
        mean(RK_var(:,k)-RV_true/T),...
        sqrt(var(RK_var(:,k)-RV_true/T)+(mean(RK_var(:,k)-RV_true/T))^2) ];
end

tblfnl = [tbl(:,[1:2:end]);tbl(:,[2:2:end])]; 
tblfnlrk = [tblrk(:,[1:2:end]);tblrk(:,[2:2:end])]; 
idx = find(tblfnl(2,6:end) == min(tblfnl(2,6:end)));
idxrk = find(tblfnlrk(2,6:end) == min(tblfnlrk(2,6:end)));
tblout = [tblfnl(:,[1,2,idx+5]),tblfnlrk(:,idxrk+5),tblfnl(:,[3,4,5])];
tblout=tblout'*100;
tblfinal = [tblfinal, tblout];

disp('PVG:')
disp(['kn: ',num2str(knl(idx))]);
disp(['hn: ',num2str(hnl(idx))]);
disp(['knp: ',num2str(knpl(idx))]);

disp('RK:')
disp(['rkn: ',num2str(rkn(idxrk))]);
disp(['jkn: ',num2str(jkn(idxrk))]);
disp(['hkn: ',num2str(hkn(idxrk))]);

 
 %% large noise results
 
 
pname =  'Results_2500';
files = dir(['./',pname,'/*.mat']);

QML_var = zeros(1000,3);
QML_qhat= zeros(1000,3);
quarticity = zeros(1000,5);
QML_std = zeros(1000,3);
QML_sstd = zeros(1000,3);
QML_ustd = zeros(1000,3);

RV_true = zeros(1000,1);
RV_hat = zeros(1000,1);
RVsub_hat = zeros(1000,1);
PVG_var = zeros(1000,27);
RK_var = zeros(1000,27);

for j=1:100

    
    y = load(['./',pname,'/',files(j).name]);
    
    QML_var  = QML_var + y.QML_var;
    QML_qhat = QML_qhat + y.QML_qhat;  
    QML_std = QML_std + y.QML_std;   
    QML_sstd = QML_sstd + y.QML_sstd; 
    QML_ustd = QML_ustd + y.QML_ustd; 
       
    RV_true = RV_true + y.RV_true;
    RVsub_hat = RVsub_hat + y.RVsub_hat;
    RV_hat = RV_hat + y.RV_hat;
    quarticity = y.results.quarticity;
    PVG_var = PVG_var+ y.PVG_var;
    RK_var = RK_var + y.RK_var;
end

T = y.T;
n=y.n;
M = y.M;


 subplot(3,3,3)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts   =   hist((QML_var(:,KK)-RV_true/T)./(QML_std(:,KK)),binCtrs);    
 nn      =   M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('large noise, using (i)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 
 subplot(3,3,9)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts=hist((QML_var(:,KK)-RV_true/T)./(QML_ustd(:,KK)),binCtrs);
 nn=M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('large noise, using (iii)','Interpreter','LaTex'); 
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 
 subplot(3,3,6)

 hold off;
 binWidth = 0.25;
 binCtrs = -5:binWidth:5;
 counts=hist((QML_var(:,KK)-RV_true/T)./(QML_sstd(:,KK)),binCtrs); 
 nn=M;
 prob = counts / (nn * binWidth);
 bar(binCtrs,prob,'hist');
 h = get(gca,'child');
 set(h,'FaceColor',[.6 .6 .6]);
 set(h,'EdgeColor',[.6 .6 .6]);
 xgrid = linspace(-4,4,81);
 hold on;
 pdfReal=pdf('Normal',-4:0.1:4,0,1);
 line(xgrid,pdfReal,'color','k','linewidth',1);
 title('large noise, using (ii)','Interpreter','LaTex');
 xlim([-4,4])
 ylim([0,0.5])
 set(gca,'fontsize',10)
 hold off;
 
 knl = y.knl;
 hnl= y.hnl;
 knpl = y.knpl;
 rkn = y.rkn;
 jkn = y.jkn;
 hkn = y.hkn;

tbl = [mean(QML_var(:,1)-RV_true/T),...
    sqrt(var(QML_var(:,1)-RV_true/T)+(mean(QML_var(:,1)-RV_true/T))^2),...
    mean(QML_var(:,2)-RV_true/T),...
    sqrt(var(QML_var(:,2)-RV_true/T)+(mean(QML_var(:,2)-RV_true/T))^2),...
    mean(RVsub_hat(:,1)/T-RV_true/T),...
    sqrt(var(RVsub_hat(:,1)/T-RV_true/T)+(mean(RVsub_hat(:,1)/T-RV_true/T))^2),...
    mean(QML_var(:,3)-RV_true/T),...
    sqrt(var(QML_var(:,3)-RV_true/T)+(mean(QML_var(:,3)-RV_true/T))^2),...
    mean(RV_hat(:,1)/T-RV_true/T),...
    sqrt(var(RV_hat(:,1)/T-RV_true/T)+(mean(RV_hat(:,1)/T-RV_true/T))^2),... 
];
tblrk = tbl;

for k=1:27
 tbl = [tbl,...
    mean(PVG_var(:,k)-RV_true/T),...
    sqrt(var(PVG_var(:,k)-RV_true/T)+(mean(PVG_var(:,k)-RV_true/T))^2) ];
end


for k=1:27
 tblrk = [tblrk,...
    mean(RK_var(:,k)-RV_true/T),...
    sqrt(var(RK_var(:,k)-RV_true/T)+(mean(RK_var(:,k)-RV_true/T))^2) ];
end

tblfnl = [tbl(:,[1:2:end]);tbl(:,[2:2:end])]; 
tblfnlrk = [tblrk(:,[1:2:end]);tblrk(:,[2:2:end])]; 
idx = find(tblfnl(2,6:end) == min(tblfnl(2,6:end)));
idxrk = find(tblfnlrk(2,6:end) == min(tblfnlrk(2,6:end)));
tblout = [tblfnl(:,[1,2,idx+5]),tblfnlrk(:,idxrk+5),tblfnl(:,[3,4,5])];
tblout=tblout'*100;
tblfinal = [tblfinal, tblout];

disp('PVG:')
disp(['kn: ',num2str(knl(idx))]);
disp(['hn: ',num2str(hnl(idx))]);
disp(['knp: ',num2str(knpl(idx))]);

disp('RK:')
disp(['rkn: ',num2str(rkn(idxrk))]);
disp(['jkn: ',num2str(jkn(idxrk))]);
disp(['hkn: ',num2str(hkn(idxrk))]);




%% Table 1

tblfinal = tblfinal([2,1,3:end],:);
 %%
 
