function param = find_initial_val(q, y)
% This version: May 24, 2017.
% MA(q) model: solve for MA parameters from autocovariances of y, gamma
% This algorithm implements Wilson (1969), which ensures invertibility of
% MA model coefficients
    n = length(y);
    param = nan(q+1,1);
    gamma = zeros(q+1,1);
    ybar  = y - mean(y);
    for i = 0:q
        
        gamma(1+i) = sum(ybar(1:(n-i)).*(ybar(i+1:n)))/n;
                
    end;
    
    
    tau = zeros(q+1,1);
    tau(1) = sqrt(gamma(1));
    eps = 1;
    tau_new  = tau;
    step  = 1;
    
    while eps>10^-6 && step < 10^3
    
        tau = tau_new;
        
        f     =   zeros(q+1,1);
       
        for j=1:q+1
            
            f(j) = sum(tau(1:q+2-j).*tau(j:q+1)) - gamma(j); 
            
        end;
        
        T = hankel(tau(1:q+1)',[tau(q+1),zeros(1,q)]) + toeplitz([tau(1),zeros(1,q)],tau(1:q+1)');
        
        tau_new = tau - T\f;

        eps = norm(tau_new - tau);    
        step  = step + 1; 
        
    end
    
    param(1)   = tau(1)^2;
    param(2:q+1) = tau(2:q+1)./tau(1);
    
    
    % check for invertbility
    
    sols = roots([param(q+1:-1:2);1]);    
    idx  = find(abs(sols)<1);
    if ~isempty(idx)
        
        sols(idx) = 1./sols(idx);
        coeffs = poly(1./sols);
        param(2:q+1) = coeffs(2:q+1);

    end
end